<?php
// This file is part of the Moodle module "EJSApp"
//
// EJSApp is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// EJSApp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle. If not, see <http://www.gnu.org/licenses/>.
//
// EJSApp has been developed by:
// - Luis de la Torre: ldelatorre@dia.uned.es
// - Ruben Heradio: rheradio@issi.uned.es
//
// at the Computer Science and Automatic Control, Spanish Open University
// (UNED), Madrid, Spain.

/**
 * Receives any .xml, .rec, .blk, .cnt, text or image file saved by an EJS applet or an EjsS javascript application.
 *
 * @package    mod_ejsapp
 * @copyright  2012 Luis de la Torre and Ruben Heradio
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once('../../config.php');
require_login();

require_once($CFG->libdir."/formslib.php");
require_once($CFG->libdir."/dml/moodle_database.php");
require_once($CFG->libdir."/blocklib.php");

// Distinguish between a file sent by EJS and EjsS.
$originalname = null;
if ($_FILES['user_file']['name'] != null) { // Receiving from EJS (java client).
    $method = true;
    $originalname = $_FILES['user_file']['name'];
    $filename = replace_characters($originalname);
} else { // Receiving from EjsS (javascript client).
    $method = false;
    $originalname = $_POST['user_file'];
    $filename = replace_characters($originalname);
    $extension = pathinfo($filename, PATHINFO_EXTENSION);
    if (!$extension) {
        if ($_POST['type'] == 'blk') {
            $extension = '.blk';
        } else if ($_POST['type'] == 'rec') {
            $extension = '.rec';
        } else if ($_POST['type'] == 'json') {
            $extension = '.json';
        } else if ($_POST['type'] == 'txt') {
            $extension = '.txt';
        } else if ($_POST['type'] == 'png') {
            $extension = '.png';
        }
        $filename = $filename . $extension;
    }
}

$contextid = $_POST['context_id'];
$userid = $_POST['user_id'];
$ejsappid = $_POST['ejsapp_id'];

// Upload the file to a temporal folder.
if ($method) { // From EJS.
    $uploaddir = $CFG->dataroot . "/tmp/";
    if (!file_exists($uploaddir)) {
        mkdir($uploaddir, 0700);
    }

    $path = $uploaddir . $filename;

    if ($originalname != null) { // As long as a file was selected...
        if (!copy($_FILES['user_file']['tmp_name'], $path)) {
            // If the file was not successfully copied, print an error message.
            echo "File $originalname could not be uploaded";
        }
    }
}

// Store the file in the user repository.

// Prepare the file info data.
$fs = get_file_storage();
// Prepare file record object.
if ($extension == 'cnt') {
    $sourceinfo = 'controller';
} else {
    $sourceinfo = 'ejsappid='.$ejsappid;
}
$fileinfo = array(
    'contextid' => $contextid, // ID of context.
    'component' => 'user', // Usually = table name.
    'filearea' => 'private', // Usually = table name.
    'itemid' => 0, // Usually = ID of row in table.
    'source' => $sourceinfo,
    'userid' => $userid,
    'filepath' => '/',
    'filename' => $filename);

// If there is an old file in the user repository with the same name, then delete it.
$oldfile = $fs->get_file($fileinfo['contextid'], $fileinfo['component'], $fileinfo['filearea'], $fileinfo['itemid'],
    $fileinfo['filepath'], $fileinfo['filename']);
if ($oldfile) {
    $oldfile->delete();
}

if ($method) { // From EJS.
    $fs->create_file_from_pathname($fileinfo, $_FILES['user_file']['tmp_name']);
    // Remove the temporal file from the temporal folder.
    unlink("$path");
} else { // From EjsS.
    if ($_POST['type'] != 'png') {
        $fs->create_file_from_string($fileinfo, rawurldecode($_POST['file']));
    } else {
        $data = rawurldecode($_POST['file']);
        list($type, $data) = explode(';', $data);
        list(, $data) = explode(',', $data);
        $data = base64_decode($data);
        $fs->create_file_from_string($fileinfo, $data);
    }
}

/**
 * Avoid problems with the file names.
 *
 * @param String $originalname
 * @return array $safefile
 */
function replace_characters($originalname) {
    $auxarray = explode('/', $originalname);
    $safefile = $auxarray[count($auxarray) - 1];
    $safefile = str_replace(" ", "_", $safefile);
    $safefile = str_replace("#", "", $safefile);
    $safefile = str_replace("$", "Dollar", $safefile);
    $safefile = str_replace("%", "Percent", $safefile);
    $safefile = str_replace("^", "", $safefile);
    $safefile = str_replace("&", "", $safefile);
    $safefile = str_replace("*", "", $safefile);
    $safefile = str_replace("?", "", $safefile);

    return $safefile;
}